@@ -24,11 +24,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import configparser
 import os
 import sys
 
-from ast import literal_eval
 from collections import defaultdict
 
 from pynicotine.events import events
@@ -58,10 +56,10 @@ class Config:
         self.set_data_folder(data_folder_path)
 
         self.config_loaded = False
-        self.parser = configparser.ConfigParser(strict=False, interpolation=None)
         self.sections = defaultdict(dict)
         self.defaults = {}
         self.removed_options = {}
+        self._parser = None
 
     @staticmethod
     def get_user_folders():
@@ -75,7 +73,7 @@ class Config:
             try:
                 data_folder_path = os.path.join(os.path.normpath(os.environ["APPDATA"]), "nicotine")
             except KeyError:
-                data_folder_path, _basename = os.path.split(sys.argv[0])
+                data_folder_path = os.path.dirname(sys.argv[0])
 
             config_folder_path = os.path.join(data_folder_path, "config")
             return config_folder_path, data_folder_path
@@ -107,7 +105,7 @@ class Config:
         """Create the folder for storing the config file in, if the folder
         doesn't exist."""
 
-        folder_path, _basename = os.path.split(self.config_file_path)
+        folder_path = os.path.dirname(self.config_file_path)
 
         if not folder_path:
             # Only file name specified, use current folder
@@ -146,6 +144,12 @@ class Config:
 
     def load_config(self):
 
+        if self.config_loaded:
+            return
+
+        from configparser import ConfigParser
+
+        self._parser = ConfigParser(strict=False, interpolation=None)
         log_folder_path = os.path.join("${NICOTINE_DATA_HOME}", "logs")
         self.defaults = {
             "server": {
@@ -395,7 +399,6 @@ class Config:
                 "dislikes": []
             },
             "players": {
-                "default": "",
                 "npothercommand": "",
                 "npplayer": "mpris",
                 "npformatlist": [],
@@ -519,10 +522,10 @@ class Config:
                 "remove_special_chars"
             ),
             "userinfo": (
-                "descrutf8"
+                "descrutf8",
             ),
             "private_rooms": (
-                "enabled"
+                "enabled",
             ),
             "logging": (
                 "logsdir",
@@ -542,7 +545,7 @@ class Config:
                 "humanizeurls"
             ),
             "notifications": (
-                "notification_tab_icons"
+                "notification_tab_icons",
             ),
             "words": (
                 "cycle",
@@ -551,17 +554,17 @@ class Config:
                 "censorfill"
             ),
             "players": (
-                "default"
+                "default",
             )
         }
 
         self.create_config_folder()
         self.create_data_folder()
 
-        load_file(self.config_file_path, self.parse_config)
+        load_file(self.config_file_path, self._parse_config)
 
         # Update config values from file
-        self.set_config()
+        self._set_config()
 
         language = self.sections["ui"]["language"]
 
@@ -575,12 +578,12 @@ class Config:
 
         events.connect("quit", self._quit)
 
-    def parse_config(self, file_path):
+    def _parse_config(self, file_path):
         """Parses the config file."""
 
         with open(encode_path(file_path), "a+", encoding="utf-8") as file_handle:
             file_handle.seek(0)
-            self.parser.read_file(file_handle)
+            self._parser.read_file(file_handle)
 
     def need_config(self):
 
@@ -590,13 +593,14 @@ class Config:
 
         return False
 
-    def set_config(self):
+    def _set_config(self):
         """Set config values parsed from file earlier."""
 
+        from ast import literal_eval
         from pynicotine.logfacility import log
 
-        for i in self.parser.sections():
-            for j, val in self.parser.items(i, raw=True):
+        for i in self._parser.sections():
+            for j, val in self._parser.items(i, raw=True):
 
                 # Check if config section exists in defaults
                 if i not in self.defaults and i not in self.removed_options:
@@ -738,8 +742,8 @@ class Config:
 
         self.config_loaded = True
 
-    def write_config_callback(self, file_path):
-        self.parser.write(file_path)
+    def _write_config_callback(self, file_path):
+        self._parser.write(file_path)
 
     def write_configuration(self):
 
@@ -748,29 +752,29 @@ class Config:
 
         # Write new config options to file
         for section, options in self.sections.items():
-            if not self.parser.has_section(section):
-                self.parser.add_section(section)
+            if not self._parser.has_section(section):
+                self._parser.add_section(section)
 
             for option, value in options.items():
                 if value is None:
                     value = ""
 
-                self.parser.set(section, option, str(value))
+                self._parser.set(section, option, str(value))
 
         # Remove legacy config options
         for section, options in self.removed_options.items():
-            if not self.parser.has_section(section):
+            if not self._parser.has_section(section):
                 continue
 
             for option in options:
-                self.parser.remove_option(section, option)
+                self._parser.remove_option(section, option)
 
         if not self.create_config_folder():
             return
 
         from pynicotine.logfacility import log
 
-        write_file_and_backup(self.config_file_path, self.write_config_callback, protect=True)
+        write_file_and_backup(self.config_file_path, self._write_config_callback, protect=True)
         log.add_debug("Saved configuration: %(file)s", {"file": self.config_file_path})
 
     def write_config_backup(self, file_path):
@@ -801,7 +805,9 @@ class Config:
 
     def _quit(self):
 
-        self.parser.clear()
+        if self._parser is not None:
+            self._parser.clear()
+
         self.sections.clear()
         self.defaults.clear()
         self.removed_options.clear()
