@@ -129,6 +129,7 @@ class Application:
 
         for action_name, callback, parameter_type, is_enabled in (
             # General
+            ("disabled", None, None, False),
             ("connect", self.on_connect, None, True),
             ("disconnect", self.on_disconnect, None, False),
             ("soulseek-privileges", self.on_soulseek_privileges, None, False),
@@ -179,7 +180,10 @@ class Application:
                 parameter_type = GLib.VariantType(parameter_type)
 
             action = Gio.SimpleAction(name=action_name, parameter_type=parameter_type, enabled=is_enabled)
-            action.connect("activate", callback)
+
+            if callback:
+                action.connect("activate", callback)
+
             self.add_action(action)
 
         self.lookup_action("away-accel").cooldown_time = 0  # needed to prevent server ban
@@ -257,9 +261,15 @@ class Application:
         ):
             self._set_accels_for_action(action_name, accelerators)
 
+        numpad_accels = []
+
         for num in range(1, 10):
+            numpad_accels.append(f"<Alt>KP_{num}")
             self._set_accels_for_action(f"win.primary-tab-{num}", [f"<Primary>{num}", f"<Alt>{num}"])
 
+        # Disable Alt+1-9 accelerators for numpad keys to avoid conflict with Alt codes
+        self._set_accels_for_action("app.disabled", numpad_accels)
+
         if GTK_API_VERSION == 3 or sys.platform != "darwin":
             return
 
@@ -281,6 +291,32 @@ class Application:
                     )
                 )
 
+            for accelerator, step, count, extend in (
+                ("<Meta>Up|<Meta>KP_Up", Gtk.MovementStep.BUFFER_ENDS, -1, False),
+                ("<Shift><Meta>Up|<Shift><Meta>KP_Up", Gtk.MovementStep.BUFFER_ENDS, -1, True),
+                ("<Meta>Down|<Meta>KP_Down", Gtk.MovementStep.BUFFER_ENDS, 1, False),
+                ("<Shift><Meta>Down|<Shift><Meta>KP_Down", Gtk.MovementStep.BUFFER_ENDS, 1, True),
+                ("<Meta>Left|<Meta>KP_Left", Gtk.MovementStep.DISPLAY_LINE_ENDS, -1, False),
+                ("<Shift><Meta>Left|<Shift><Meta>KP_Left", Gtk.MovementStep.DISPLAY_LINE_ENDS, -1, True),
+                ("<Meta>Right|<Meta>KP_Right", Gtk.MovementStep.DISPLAY_LINE_ENDS, 1, False),
+                ("<Shift><Meta>Right|<Shift><Meta>KP_Right", Gtk.MovementStep.DISPLAY_LINE_ENDS, 1, True),
+                ("<Alt>Left|<Alt>KP_Left", Gtk.MovementStep.WORDS, -1, False),
+                ("<Shift><Alt>Left|<Shift><Alt>KP_Left", Gtk.MovementStep.WORDS, -1, True),
+                ("<Alt>Right|<Alt>KP_Right", Gtk.MovementStep.WORDS, 1, False),
+                ("<Shift><Alt>Right|<Shift><Alt>KP_Right", Gtk.MovementStep.WORDS, 1, True)
+            ):
+                widget.add_shortcut(
+                    Gtk.Shortcut(
+                        trigger=Gtk.ShortcutTrigger.parse_string(accelerator),
+                        action=Gtk.SignalAction(signal_name="move-cursor"),
+                        arguments=GLib.Variant.new_tuple(
+                            GLib.Variant.new_int32(step),
+                            GLib.Variant.new_int32(count),
+                            GLib.Variant.new_boolean(extend)
+                        )
+                    )
+                )
+
     def _update_user_status(self, *_args):
 
         status = core.users.login_status
@@ -1003,13 +1039,13 @@ class Application:
         if self.wishlist is not None:
             self.wishlist.destroy()
 
-        if self.tray_icon is not None:
-            self.tray_icon.destroy()
-
         if self.spell_checker is not None:
             self.spell_checker.destroy()
 
         if self.window is not None:
             self.window.destroy()
 
+        if self.tray_icon is not None:
+            self.tray_icon.destroy()
+
         self.__dict__.clear()
