@@ -86,6 +86,7 @@ class UserBrowses(IconNotebook):
         for event_name, callback in (
             ("peer-connection-closed", self.peer_connection_error),
             ("peer-connection-error", self.peer_connection_error),
+            ("quit", self.quit),
             ("server-disconnect", self.server_disconnect),
             ("server-login", self.on_focus),
             ("shared-file-list-progress", self.shared_file_list_progress),
@@ -96,6 +97,9 @@ class UserBrowses(IconNotebook):
         ):
             events.connect(event_name, callback)
 
+    def quit(self):
+        self.freeze()
+
     def destroy(self):
 
         self.userbrowse_combobox.destroy()
@@ -287,21 +291,21 @@ class UserBrowse:
                 ("#" + _("F_ile Properties"), self.on_file_properties, True),
                 ("", None),
                 ("#" + _("Copy _Folder Path"), self.on_copy_folder_path),
-                ("#" + _("Copy _URL"), self.on_copy_folder_url),
+                ("#" + _("Copy Folder U_RL"), self.on_copy_folder_url),
                 ("", None),
-                (">" + _("User"), self.user_popup_menu)
+                (">" + _("User Actions"), self.user_popup_menu)
             )
         else:
             self.folder_popup_menu.add_items(
-                ("#" + _("Download Folder & Subfolders"), self.on_download_folder_recursive),
-                ("#" + _("Download Folder & Subfolders To…"), self.on_download_folder_recursive_to),
+                ("#" + _("_Download Folder & Subfolders"), self.on_download_folder_recursive),
+                ("#" + _("Download Folder & Subfolders _To…"), self.on_download_folder_recursive_to),
                 ("", None),
                 ("#" + _("F_ile Properties"), self.on_file_properties, True),
                 ("", None),
                 ("#" + _("Copy _Folder Path"), self.on_copy_folder_path),
-                ("#" + _("Copy _URL"), self.on_copy_folder_url),
+                ("#" + _("Copy Folder U_RL"), self.on_copy_folder_url),
                 ("", None),
-                (">" + _("User"), self.user_popup_menu)
+                (">" + _("User Actions"), self.user_popup_menu)
             )
 
         # Setup file_list_view
@@ -367,7 +371,7 @@ class UserBrowse:
                 ("#" + _("Copy _File Path"), self.on_copy_file_path),
                 ("#" + _("Copy _URL"), self.on_copy_url),
                 ("", None),
-                (">" + _("User"), self.user_popup_menu)
+                (">" + _("User Actions"), self.user_popup_menu)
             )
         else:
             self.file_popup_menu.add_items(
@@ -382,7 +386,7 @@ class UserBrowse:
                 ("#" + _("Copy _File Path"), self.on_copy_file_path),
                 ("#" + _("Copy _URL"), self.on_copy_url),
                 ("", None),
-                (">" + _("User"), self.user_popup_menu)
+                (">" + _("User Actions"), self.user_popup_menu)
             )
 
         # Key Bindings (folder_tree_view)
@@ -546,12 +550,7 @@ class UserBrowse:
         self.search_entry.set_text("")
         self.search_button.set_active(False)
 
-        try:
-            folder_path, basename = self.queued_path.rsplit("\\", 1)
-
-        except ValueError:
-            folder_path = basename = None
-
+        folder_path, _separator, basename = self.queued_path.rpartition("\\")
         iterator = self.folder_tree_view.iterators.get(folder_path)
 
         if not iterator:
@@ -725,6 +724,12 @@ class UserBrowse:
         if self.active_folder_path == folder_path:
             return
 
+        browsed_user = core.userbrowse.users.get(self.user)
+
+        if browsed_user is None:
+            # Redundant row selection event when closing tab, prevent crash
+            return
+
         self.populate_path_bar(folder_path)
         self.file_list_view.clear()
 
@@ -733,7 +738,6 @@ class UserBrowse:
         if not folder_path:
             return
 
-        browsed_user = core.userbrowse.users[self.user]
         files = browsed_user.public_folders.get(folder_path)
 
         if not files:
@@ -1430,7 +1434,8 @@ class UserBrowse:
         self.set_in_progress()
 
         if self.local_permission_level:
-            core.userbrowse.browse_local_shares(permission_level=self.local_permission_level, new_request=True)
+            core.userbrowse.browse_local_shares(
+                path=file_path, permission_level=self.local_permission_level, new_request=True)
         else:
             core.userbrowse.browse_user(self.user, path=file_path, new_request=True)
 
